use Getopt::Long;
use File::Copy;

$correctname=substr($0,0);
#print "\$correctname = $correctname\n";

use lib "/mt/apps/gpms/DateTrans/lib";
use DateTrans qw(monName monNumber timeFormat AmPmFormat dayNumbeFormat);

use lib "/mt/apps/gpms/DateTrans/lib";
use  xsdxsltcheck qw(xsdcheck xsltcheck FGDC2MRFcheck xsl4mrf2fgdccheck);

use File::Basename;

sub usage() {
    print "Usage: perl $correctname [-listfile=<listfile>][-directory=<directory>][-product=<product>]\n";
    print "Example:\n";
    print "perl $correctname -listfile=list.txt -directory=/dir/subdir -product=TLFE2007\n";
    print"where list.txt is the filename of the list file containg the names of the files to validated,\n";
       print "/dir/subdir is the directory containing the files to be validated\n";
       print "and TLFE2007 is the product.\n";
    print "The following keyword shortcuts are also available for the directory:\n";
       print "mtdata for /mtdata/geo/gpms/input/fgdc\n";
       print "nfs for /nfs/gpms/input/fgdc\n";
       print "prodmtdata for /prodmtdata/geo/gpms/input/fgdc\n";

}
%gOptions;
&GetOptions(\%gOptions, qw(listfile=s directory=s  product=s));
unless ($gOptions{listfile} and $gOptions{directory} and $gOptions{product}) {
        usage();
        exit(1);
};

#-----------------------------------------------Notes Section-------------------------------------------------------------------------------------------
#Written by Matthew McCready
#This script takes a FGDC file and transforms it into an ISO 19115, 19139 and 19110 file.

# 12/16/2011: Added the capability to process Entity and Attribute Information. This script will produce an Entity and Attribute file for every file.
#This scripts check if the EA file exists. If the EA file does not exist, it creates one. The script will then check for the gfc:featureType element. 
# If this element is not in the file, it is deleted.

#2/24/2012 Added the "opening the file to insert an updated value for gml:TimePeriod" section. This was done so that the gml:id attribute of the 
#gml:TimePeriod element will have a unique value. This was acomplished by reading in the file and searching for the line that contains the  gml:TimePeriod
#element. When this line is found, the contents of the gml:id attribute are seperated out and a value is added to it based based on an updated counter 
#variable. The line is then concatenated back together to a new file. All the other lines copied over to the new file. The original is then removed and
#the new file is then copied over to the original so that the new file has the same filename as the original file.  
 
#---------------------------------------------file mangement section----------------------------------------------------------------------------------------
$Infile = $gOptions{listfile};
chomp($Branch = $gOptions{product});
chomp($indir= $gOptions{directory});

#enabling keywords for the input directory
if ($indir eq "mtdata"){
$indir = "/mtdata/geo/gpms/input/fgdc";
}elsif ($indir eq "nfs"){
$indir = "/nfs/gpms/input/fgdc";
}elsif ($indir eq "prodmtdata"){
$indir = "/prodmtdata/geo/gpms/input/fgdc";
}elsif ($indir eq "GPMSscripts"){
$indir = "/home/mccre004/GPMSscripts";
};

$root = "/mt/apps";
$rootout="/mtdata/geo";

$inlist="$rootout/gpms/input/list";
$outputdir ="$rootout/gpms/output/mrf";
$errlogdir ="$rootout/gpms/log";
$combolist = $inlist . "/" . $Infile;
$fgdcdir = "$rootout/gpms/output/fgdc";
$stylesheet ="$root/gpms/FGDC2MRF/MIF2MRF.xsl";
$ISOdir="$rootout/gpms/output/ISO";
#ISOEADir = "$ISOdir/EAFiles";
$newdir=$Branch . $dateString;

#-----------------------------------------------------counter variable mangement section--------------------------------------------------------------------
$invalidcounter=0;
$badxmlcounter=0;
$nopostcounter=0;
$fgdcoutcounter=0;
$fgdcoutcountera=0;
$flag_array_countera=0;
$postarraycounter=0;
$passcounter=0;
$mifarraycounter=0;
#$gmlUpdateVariable=1;
#------------------------------------------------------error trap--------------------------------------------------------------------------------------------
&xsdcheck;
&xsltcheck;
&FGDC2MRFcheck;
&xsl4mrf2fgdccheck;

if (-e $indir){
print "Please Wait, Now processing the files ----------------------------------------------\n\n";
}else{
print "The directory $indir  could not be found! \n";
print "Please make sure the pathname, $indir, was spelled correctly and that it exists.\n";
exit(1);
};

if (-s $indir){
}else{
print "The directory $indir does not contain any files to be validated! \n";
exit(1);
};

if (-r $indir){
}else{
print "The directory $indir is not readable!!!!!!!!!!!!!!!!!!!!!\n";
print "Please make sure you have the permissions to read the directory. \n";
exit(1);
};

if (-w $indir){
#print "The file $combolist has files\n";
}else{
print "The directory $indir is not writeable!!!!!!!!!!!!!!!!!!!!!\n";
print "Please make sure you have the permissions to write to the directory. \n";
exit(1);
};

#$outputdir
if (-e $outputdir){
}else{
print "The directory $outputdir  could not be found! \n";
print "Please make sure the pathname, $outputdir, was spelled correctly and that it exists.\n";
exit(1);
};

if (-w $outputdir){
}else{
print "The directory $outputdir can not be written to!!!!!!!!!! \n";
print "Please make sure that you have the permissions needed to write to the directory.\n";
exit(1);
};

if (-e $combolist){
}else{
print "The file $combolist could not be found! \n";
print "Please make sure the file, $Infile,  was spelled correctly and that the file is in the $inlist directory.\n";
exit(1);
};

#making sure the file list contains something

if (-s $combolist>2){
}else{
$size=-s $combolist;
print "The file $combolist does not contain the list of files to be validated! (It has a size of $size kb) \n";
print "Please make sure the file, $Infile,  contains a list of files to be validated.\n";
exit(1);
};

#-----------------------------------redirecting the output---------------------------------------------------------------------------------------------------
open (SAVEOUT,">&STDOUT");
#------------------------------------getting the date---------------------------------------------------------------------------------------------------------

my $now=`date`;
$mon = substr($now,4,3);
$day=substr($now,8,2);
$year =substr($now,24,4);
$hour = substr($now,11,2);
$min = substr($now,14,2);
my $daynum= dayNumbeFormat($day);

#converting the date to a number
my $monName=monName($mon);
my $monnum =monNumber($mon);

$dateString =$year . $monnum . $daynum . $hour . $min;

$time = $hour . ":" . $min;
$oldhour = $hour;
my $newhour =timeFormat($hour);
my $ampm =AmPmFormat($oldhour);


$newtime = $newhour . ":" . $min . " $ampm";

#-----------------------------------------making the directories------------------------------------------------
$newdir=$Branch . $dateString;
$newerrdir ="$errlogdir/$newdir";
system "mkdir $newerrdir";
system "mkdir $ISOdir/$newdir";
chmod 0777, "$newerrdir";
chmod 0777, "$fgdcdir/$newdir";
chmod 0777, " $ISOdir/$newdir";
$ISOEADir="$ISOdir/$newdir/EAFiles";
system "mkdir $ISOEADir";
#-----------------------------------------Filename array creation section------------------------------------------------------------------------------
#open a filehamdle for the file containing the filenames
$flag_array_counter=0;
open INPUTFILE, "$combolist";
while(<INPUTFILE>){

$slashpos= rindex($_, "/");
#print "getting slash position--------------\n";
if ($slashpos < 0){


$filenameflag=$_;
$noflagarray[$flag_array_counter]=$filenameflag;
$flag_array_counter++;
$flagpos=rindex($_,".");
$flagless=substr($_, 0,$flagpos+4);
$finalarray[$finalarraycounter]=$flagless;
$finalarraycounter++;

}elsif($slashpos > 0){

#for filenames with directories
my $basename = basename $_;
my $dirnameA = dirname $_;

$slash_directory_name[$slash_directory_name_counter]=$dirnameA;
$slash_directory_name_counter++;

$flagpos=rindex($_,".");
#print "\$flagpos = $flagpos\n";
$flagless=substr($_, 0,$flagpos+4);#was 3
};
};


#----------------------------------------------------------Saxon-----------------------------------------------------------------------------------------
#$ENV{'PATH'} = "/opt/saxon_lib/jdk1.5.0_11/bin:/mt/apps/gpms/output/mrf$newerrdir:$ENV{'PATH'}";#goodone
#$cp = "CLASSPATH";#goodone
#$ENV{$cp} = "/opt/saxon_lib/jdk1.5.0_11/lib/:/opt/saxon_lib/saxon9/saxon9sa.jar:/opt/saxon_lib/saxon9/saxon9-jdom.jar:/opt/saxon_lib/saxon9/:/opt/saxon_lib/saxon9/saxon9.jar:$ENV{$cp}";

$ENV{'PATH'} = "/apps/saxon/jdk1.5.0_24/bin::/apps/saxon/saxon9ee.jar::/apps/saxon:$rootout/mrf/$newerrdir:$ENV{'PATH'}";

$cp = "CLASSPATH";#goodone
$ENV{$cp} = "/apps/saxon/jdk1.5.0_24/lib/:/apps/saxon/saxon9ee.jar:/apps/saxon/saxon9ee.jar $CLASSPATH:/usr/share/java/saxon.jar";
#---------------------------------------------------transforming to ISO Standard------------------------------------------------------------------------

print SAVEOUT "Now Transforming the files to ISO 19115 format-----------------------------------------------------------------------------\n";

foreach $finalarray  (@finalarray){
$finalarrayCounter++;
print SAVEOUT "now reading $finalarray\n";
#/mt/apps/gpms/IsoSchema

$iso_var= "ISO_" . $finalarray;

#$iso_var_ea= "ISO_EA_" . $finalarray;
#print "\$iso_var = $iso_var\n"; 

open STDOUT,">$iso_var";
open STDERR, ">$iso_var.err";
#rint SAVEOUT "\

my @csax = "java com.saxonica.Transform -outval:recover  $indir/$finalarray  $root/gpms/IsoSchema/csdgm2iso19115-2_master.xslt";
my $mif = system(@csax);

#print SAVEOUT "\$mif = $mif\n";

if ($mif == 0){
#print SAVEOUT "Good Transformation\n";
$NewISOFileArray[$NewISOFileArrayCounter] = $iso_var;
$NewISOFileArrayCounter++;
system "rm $iso_var.err";

}else{
#print SAVEOUT "Here for the errors!!!!!!!!!\n";
system "mv $iso_var $newerrdir";
system "mv $iso_var.err $newerrdir";
$badFile[$badFileCounter] =$finalarray;
$badFileCounter++; 
};

} ;


#-----------------------------------------opening the file to inserted an updated value for gml:TimePeriod --------------------------------------------------------------
print SAVEOUT "\n";
foreach $NewISOFileArray (@NewISOFileArray){ 
$gmlUpdateVariable =1;
#print SAVEOUT " --------------------------------------------------------------\n";
print SAVEOUT "Now correcting $NewISOFileArray ------------------\n";
system "chmod 777 $NewISOFileArray"; 

$iso_varDotLocation=index($NewISOFileArray,".");
$iso_var_pre=substr($NewISOFileArray,0, $iso_varDotLocation);
#print SAVEOUT "\$iso_var_pre = $iso_var_pre\n";
$new_iso_var= $iso_var_pre . "_corrected.shp.xml";
#print SAVEOUT "\$new_iso_var = $new_iso_var\n";
open ISOFILE, "<$NewISOFileArray";
my (@lines) = <ISOFILE>; 
open NEWISOFILE, ">$new_iso_var";


#while (<ISOFILE>){
foreach $line (@lines){ # loop thru list
#print SAVEOUT "\$_ = $_\n";
#my ($line)=$_;
if($line =~ /gml:TimePeriod xmlns:gml/){
#print SAVEOUT "IN the correction Zone!!!!!!!!!!!\n";
$gmlIDLocation=index($line,"gml:id");
$gmlPreId = substr($line,0,$gmlIDLocation-1);
$PostgmlID=substr($line,$gmlIDLocation);
$BracketLocation=index($PostgmlID,">")-1;
$gmlID=substr($PostgmlID,0,$BracketLocation);

#print SAVEOUT "\$gmlID = $gmlID\n";
$newGMLID = $gmlID . $gmlUpdateVariable;


$newLine= $gmlPreId . " " . $newGMLID . "\">";
#print SAVEOUT "\$newLine = $newLine\n";
print NEWISOFILE "$newLine";
$gmlUpdateVariable++;
}elsif($line =~ /gmi:MI_Metadata/){
print NEWISOFILE "$line";
#print SAVEOUT "$line";

}else{
#print SAVEOUT "IN the default zone!!!!!!!!!!\n";
print NEWISOFILE "$line";
#print SAVEOUT "$line";


}; 
close ISOFILE;
};
#system 

system "rm $NewISOFileArray";
system "cp $new_iso_var $NewISOFileArray";  
#system" mv $new_iso_var $ISOdir/$newdir";
system "mv $NewISOFileArray $ISOdir/$newdir";
system "rm $new_iso_var";
};
#-----------------------------------------------------------------------------------------------------------------------------------------------------------------------




#####-----------------------------------for the entity and attributes --------------------------------------------------------------------------------------
print SAVEOUT "\nNow making the Attribute and Info XSLT by transforming the files into the ISO 19110 format\n"; 
foreach $finalarray  (@finalarray){
print SAVEOUUT "\$finalarray  = $finalarray \n";
#----------------------------------------determing the filetype---------------------------------------------------------------------------------------------

$tigerOrgz =substr($finalarray,0,2);
$PosBegDash=index($finalarray,"_")+1;
$FileYear=substr($finalarray,$PosBegDash,4);

if ($tigerOrgz eq "tl"){ 
$posofDot=index($finalarray,".");
$posofDash=rindex($finalarray,"_")+1;
$fileTypeLength= $posofDot-$posofDash;
$filetype=substr($finalarray,$posofDash,$fileTypeLength);
$iso_var_ea= "ISO_EA_" . $FileYear . "_" . $filetype . ".xml";
$iso_var_ea_error = "ISO_EA_" . $FileYear . "_" . $filetype . ".err";
}elsif($tigerOrgz eq "gz"){

$LocPostYear=$PosBegDash+5;
$postYear=substr($finalarray,$LocPostYear);
$gzFirstDash=index($postYear,"_")+1;
$filetype=substr($postYear,$gzFirstDash,3);
$iso_var_ea= "ISO_EA_" . $FileYear . "_" . $filetype . ".xml";
$iso_var_ea_error = "ISO_EA_" . $FileYear . "_" . $filetype . ".err";
}else{
open UNIDTHEME, ">>$newerrdir/$logname/UnknownThemes.txt";
$UnIdThemeArray[$UnIdThemes]=$finalarray;
$UnIdThemes++;
print UNIDTHEME "$finalarray\n";
close UNIDTHEME;
};
$testDir = $ISOEADir . "/" . $iso_var_ea;

if (-e $testDir){
system "rm $iso_var_ea_error";
}else{
print SAVEOUT "now making the Entity and Attribute file $iso_var_ea-------------------------------\n";


open STDOUT,">$iso_var_ea";
open STDERR, ">$iso_var_ea_error";
my @csaxea = "java com.saxonica.Transform -outval:recover  $indir/$finalarray  $root/gpms/IsoSchema/Entity-n-Attributes-to-Feature-Catalog.xslt";#Sec52iso.xslt
my $mifea = system(@csaxea);
 
if ($mifea == 0){
#print SAVEOUT "Good EA\n";
open EAFILE, "<$iso_var_ea";

while (<EAFILE>){
$line=$_;

if ($line =~/gfc:featureType/){
$goodEAFile="yes";
$EAArray[$EAFileCounter]= $iso_var_ea;
$EAFileCounter++;
close EAFILE;
last;
}elsif($line =~/\/gfc:FC_FeatureCatalogue/){
$goodEAFile="no";
close EAFILE;
last;
};

};

if ($goodEAFile eq "yes"){
system "mv $iso_var_ea $ISOEADir";
system "rm $iso_var_ea_error";
close STDOUT;
}else{
system "rm $iso_var_ea $ISOEADir";
system "rm $iso_var_ea_error";
};

}else{ 
#print SAVEOUT "EA Error!!!!!!!!!!!!!!!!\n";
system "mv $iso_var_ea $newerrdir";
system "mv $iso_var_ea_error $newerrdir";
$badFile[$badFileCounter] =$finalarray;
$badFileCounter++;
}; 
};



};
#-----------------------------------getting the total number of files------------------------------------------------------------------------------------------
print SAVEOUT "Now calculating the total number of files-------------------------------------------------------------------------------\n\n";
$total=$finalarrayCounter+$goodcounter+$badFileCounter;

if ($finalarrayCounter>0){
$perTrans=($finalarrayCounter/$total)*100;
}else{
$perTrans=0;
};

if ($badFileCounter>0){
$perBad=($badFileCounter/$total)*100
}else{
$perBad=0;
};

#--------------------------setting up the log file-------------------------------------------------------------------------------------------------------------
$logname="$newdir.log";

if ($badFileCounter < 1){
#print SAVEOUT "resetting the variable\n";
$badFileCounter="0";
};

open LOG,">$newerrdir/$logname";

print LOG "This log details the transformation of $total the FGDC files in the $Infile list file to the ISO 19115-2 format. \n ";
print LOG "These files can be found in the $indir directory. ";
print LOG "It was run on $monnum/$daynum/$year at $time ($newtime) for the $Branch product.\n\n"; 

print LOG "Of these $total files, $finalarrayCounter, or $perTrans%, were successfully transformed into the ISO format and $badFileCounter, or $perBad%, were not ";
print LOG "transformed because of errors in the FGDC file.\n";

if ($EAFileCounter >0){
print LOG "For the $total FGDC files, $EAFileCounter Entity and Attribute, or ISO 19110, files were created. One entity and attribute file was created for every file type with";
print LOG " Entity and attribute information.\n";
#print LOG "Please see the lists of files below:\n\n";
#}else{

#print LOG "Please see the lists of files below:\n\n";
};

if ($UnIdThemes >0){

if ($UnIdThemes == 1){
$UnIdThemesNoun= "file"
}else{
$UnIdThemesNoun= "files"
};

print LOG "Also, see the UnknownThemes.txt file for the $UnIdThemes $UnIdThemesNoun whose theme could not be identified.\n"
};

print LOG "\nPlease see the lists of files below:\n\n";
#print LOG "\n New !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
#print SAVEOUT "\n New !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";

#--------------------------indicator section ----------------------------------------------------------------------------------------------------------------------

if ( $finalarrayCounter >1){
print SAVEOUT "The following files were transformed into an ISO 19115-2 format metadata files. Please see them in the $ISOdir/$newdir directory.\n"; 
print LOG "The following files were transformed into an ISO 19115-2 format metadata files. Please see them in the $ISOdir/$newdir directory. \n";
}elsif ($finalarrayCounter == 1){
print SAVEOUT "The following file was transformed into an ISO 19115-2 format metadata file. Please see it in the $ISOdir/$newdir directory.\n";
print LOG "The following file was transformed into an ISO 19115-2 format metadata file. Please see it in the $ISOdir/$newdir directory.\n";
};

$goodcounter=1;
foreach $finalarray  (@finalarray){
print SAVEOUT "$goodcounter:  $finalarray\n";
print LOG "$goodcounter:  $finalarray\n";
$goodcounter++;
};

if ($EAFileCounter >0){
print SAVEOUT "\n \n The following Entity and Attribute files have been created. Please see them in the $ISOEADir directory.\n";
print LOG  "\n \n The following Entity and Attribute files have been created. Please see them in the $ISOEADir directory.\n";
$EAFileCounterB=1;
foreach  $EAArray (@EAArray){
print SAVEOUT "$EAFileCounterB: $EAArray\n";
print LOG "$EAFileCounterB: $EAArray\n";
$EAFileCounterB++;
};
};

if ($badFileCounter > 1){
print SAVEOUT "\nThe following files could not be transformed into an ISO 19115 format metadata files because they contain errors. Please see the error files and the original xml files  at $newerrdir.\n";
print LOG "\nThe following files could not be transformed into an ISO 19115 format metadata files because they contain errors. Please see the error files and the original xml files at $newerrdir.\n";
}elsif($badFileCounter ==1){
print SAVEOUT "\nThe following file could not be transformed into an ISO 19115 format metadata file because it contains errors. Please see the error file at and the original xml file $newerrdir.\n";
print LOG "\nThe following file could not be transformed into an ISO 19115 format metadata file because it contains errors. Please see the error file and the original xml file at $newerrdir.\n.";
};

$badcounter=1;
foreach $badFile (@badFile){
print SAVEOUT "$badcounter:$badFile\n";
print LOG "$badcounter:$badFile\n";
$badcounter++;
};

if ($UnIdThemes >0){
if ($UnIdThemes >1){
print SAVEOUT "\nThe theme for the following files could not be identified. Please see the file of unknown themes (UnknownThemes.txt) at $newerrdir:\n";
print LOG "\nThe theme for the following files could not be identified. Please see the file of unknown themes (UnknownThemes.txt) at $newerrdir: \n";
}elsif($UnIdThemes ==1){
print SAVEOUT "\nThe theme for the following file could not be identified. Please see the file of unknown themes (UnknownThemes.txt) at $newerrdir:\n";
print LOG "\nThe theme for the following file could not be identified. Please see the file of unknown themes (UnknownThemes.txt) at $newerrdir:\n";
};
$MissingThemeCounter=1;
foreach $UnIdThemeArray (@UnIdThemeArray){
print SAVEOUT "$MissingThemeCounter: $UnIdThemeArray\n";
print LOG "$MissingThemeCounter: $UnIdThemeArray\n";
$MissingThemeCounter++;
};
};

print SAVEOUT "\n";
